# -*-python-*-
# $Id: simplegps.py,v 1.6 2006/10/05 01:16:55 asc Exp $

__package__    = "simplegps"
__version__    = "1.2"
__author__     = "Aaron Straup Cope"
__url__        = "http://www.aaronland.info/python/simplegps/"
__cvsversion__ = "$Revision: 1.6 $"
__date__       = "$Date: 2006/10/05 01:16:55 $"
__copyright__  = "Copyright (c) 2006 Aaron Straup Cope. Perl Artistic License."

import lightblue
import simplegpsutils

class simplegps (simplegpsutils.simplegpsutils) :

    def __init__ (self, service=None) :
        self.__service = service        
        self.__socket  = None

    #
    #
    #

    def __del__ (self) :
        if self.__socket != None :
            self.disconnect_gps_device()            

        if not self.__service :
            self.__service = None
            
    #
    #
    #

    def service (self) :
        return self.__service

    #
    #
    #
    
    def pair_gps_device (self) :

        if self.__service :
            return True

        try :
            service = lightblue.selectservice()
        except Exception, e :
            return False
        
        if service == None :
            return False
        
        self.__service = service
        return True
    
    #
    #
    #
    
    def connect_gps_device (self) :

        if self.__socket != None :
            return True

        if not self.pair_gps_device() :
            return False

        try :
            self.__socket = lightblue.socket()
        except Exception, e :
            return False

        try :
            self.__socket.connect((self.__service[0], self.__service[1]))
        except Exception, e :
            return False
        
        return True

    #
    #
    #
    
    def poll (self, attempts=1, raw=False) :

        if not self.connect_gps_device() :
            return None

        #

        data  = None
        tried = 1

        while tried <= attempts and data == None :
            # print "%s/%s" % (tried, attempts)
            
            data = self.poll_gprmc_data()
            tried += 1

        self.disconnect_gps_device()
        
        #

        # print "raw : %s" % data
        
        if raw :
            return data
        
        lat = None
        lon = None
        ts  = None
        
        if data != "" :

            try :
                (lat, lon) = self.parse_gps_data(data)                
            except Exception, e :
                return None

        if not lat or not lon :
            return None
        
        #

        return (lat, lon)

    #
    #
    #
    
    def parse_gps_data (self, gps_data) :

        # Thank you :
        # http://positio.rista.net/en/pys60gps/
        
        # Expect 11 ','-delimiters (NMEA 2.2)
        # Expect 12 ','-delimiters (NMEA 2.3)

        lat = ''
        lon = ''
        
        if (gps_data.count(",") == 11) :
            (GPRMC,hhmmssss,status,lat,l1,lon,l2,speed_knots,course,ddmmyy,mvd,mdr_cs)=gps_data.split(",")
        elif (gps_data.count(",") == 12) :
            (GPRMC,hhmmssss,status,lat,l1,lon,l2,speed_knots,course,ddmmyy,mvd,mdr,fix_cs)=gps_data.split(",")
        else:
            pass

        if (status is 'A' and lat is not "" and lon is not ""):
            lat, lon = self.parse_gps_latlong(lat, l1, lon, l2)

        # print "* %s %s" % (lat, lon)
        return (lat, lon)

    #
    #
    #

    def parse_gps_latlong (self, lat, l1, lon, l2):
        
        try:
            latitude  = float(lat[0:2]) + (float(lat[2:4] + "." + lat[5:9])/60)
            longitude = float(lon[0:3]) + (float(lon[3:5] + "." + lon[6:10])/60)

            if l1 is "S":
                latitude = -latitude

            if l2 is "W":
                longitude = -longitude

            latitude  = float("%.6f" % latitude)
            longitude = float("%.6f" % longitude)
            
            return latitude, longitude
        except:
            return (None, None)
        
    #
    #
    #

    def poll_gprmc_data (self) :
        return self.poll_gps_data("$GPRMC")

    #
    #
    #
    
    def poll_gps_data (self, type, iterations=50) :
        
        count = 1
        data  = ""
        
        while count < iterations :
            
            try :
                str = self.__socket.recv(512)
            except Exception, e :
                return None

            lines = str.split("\r\n")

            for ln in lines :
                if ln.startswith(type) :
                    return ln

            count += 1
            
        return None

    #
    #
    #

    def disconnect_gps_device (self) :
        self.__socket.close()
        self.__socket = None

    #
    #
    #

    
if __name__ == "__main__" :

    gps    = simplegps()
    coords = gps.poll(2)
    
    if not coords :
        print "Ack! Failed to make happy with your GPS device!!"
    else :

        lat = coords[0]
        lon = coords[1]
        
        print "%s %s" % (lat, lon)
        print gps.prettyprint_latlon(lat, lon)
        
        #
    
        (swlat, swlon, nelat, nelon) = gps.mk_bounding_box(lat, lon, 5)
        print "%s %s / %s %s" % (swlat, swlon, nelat, nelon)

        #
        
        ob_lat = float(37.769671)
        ob_lon = float(-122.511951)
        
        d1 = gps.calculate_distance(lat, lon, ob_lat, ob_lon, 'km')
        d2 = gps.calculate_distance(lat, lon, ob_lat, ob_lon, 'miles')    
        
        print "%s km to Ocean Beach, SF CA" % d1
        print "%s miles to Ocean Beach, SF CA" % d2
