# -*-python-*-
# $Id: simplegpsutils.py,v 1.1 2006/10/05 01:16:55 asc Exp $

__package__    = "simplegps"
__version__    = "1.2"
__author__     = "Aaron Straup Cope"
__url__        = "http://www.aaronland.info/python/simplegps/"
__cvsversion__ = "$Revision: 1.1 $"
__date__       = "$Date: 2006/10/05 01:16:55 $"
__copyright__  = "Copyright (c) 2006 Aaron Straup Cope. Perl Artistic License."

import math

class simplegpsutils :

    def __init__ (self) :
        pass
    
    #
    #
    #
    
    def mk_bounding_box (self, lat, lon, dist, measure='km') :
        lat  = float(lat)
        lon  = float(lon)
        dist = float(dist)
        
        r = self.earths_radius(measure)
        
        rad  = self.rad(1)
        dlat = (dist / r) * rad
        
        swlat = lat - dlat
        nelat = lat + dlat
        
        swlon = lon - ((dist * math.cos(swlat) / r) * rad)
        nelon = lon + ((dist * math.cos(nelat) / r) * rad)
        
        return (swlat, swlon, nelat, nelon)
    
    #
    #
    #
    
    def calculate_distance (self, from_lat, from_lon, to_lat, to_lon, measure='km') :
        
        from_lat = self.rad(float(from_lat))
        from_lon = self.rad(float(from_lon))
        to_lat   = self.rad(float(to_lat))
        to_lon   = self.rad(float(to_lon))
        
        r = self.earths_radius(measure)
        
        dist = math.acos(math.sin(from_lat) * math.sin(to_lat) + math.cos(from_lat) * math.cos(to_lat) * math.cos(to_lon-from_lon)) * r
        return float("%.2f" % dist)
    
    #
    #
    #
    
    def prettyprint_latlon (self, lat, lon) :
        lat = float(lat)
        lon = float(lon)
        
        bear_lat = ""
        bear_lon = ""
        
        if lat < 0 :
            bear_lat = "S"
        elif lat > 0 :
            bear_lat = "N"
        else :
            pass
        
        if lon < 0 :
            bear_lon = "W"
        elif lon > 0 :
            bear_lon = "E"
        else :
            pass
            
        lat_dms = self.decimal_to_dms(lat)
        lon_dms = self.decimal_to_dms(lon)
        
        lat_str = "%d %d %.2f %s" % (lat_dms[0], lat_dms[1], lat_dms[2], bear_lat)
        lon_str = "%d %d %.2f %s" % (lon_dms[0], lon_dms[1], lon_dms[2], bear_lon)        
        
        return "%s %s" % (lat_str, lon_str)

    #
    #
    #
    
    def decimal_to_dms (self, dec) :
        
        deg = int(dec);
        
        dec_min = abs(dec - deg) * 60
        min     = int(dec_min)
        
        sec = (dec_min - min) * 60
        sec = float("%.2f" % sec)
        
        return (deg, min, sec)
    
    #
    #
    #
    
    def rad (self, v) :
        return float(v) * (math.pi / float(180))
    
    #
    #
    #
    
    def earths_radius (self, measure='km') :
        
        measure = measure.lower()
        
        if measure == 'miles' :
            return float(3963.1)
        else :
            return float(6371)            
    
if __name__ == "__main__" :

    u = simplegpsutils()
    print "%s" % u.earths_radius()
